#!/usr/bin/env python

"""
Pandoc filter to convert divs with latex="true" to LaTeX
environments in LaTeX output. The first class
will be regarded as the name of the latex environment
e.g.
<div latex="true" class="note abc">...</div>
will becomes
\begin{note}...\end{note}
"""

from pandocfilters import toJSONFilter, RawBlock, Div


def latex(x):
    return RawBlock('latex', x)


def latexdivs(key, value, format, meta):
    if key == 'Div':
        [[ident, classes, kvs], contents] = value
        if ["latex","true"] in kvs:
            if format == "latex":
                if ident == "":
                    label = ""
                else:
                    label = '\\label{' + ident + '}'
                return([latex('\\begin{' + classes[0] + '}' + label)] + contents +
                       [latex('\\end{' + classes[0] + '}')])

if __name__ == "__main__":
    toJSONFilter(latexdivs)



#!/usr/bin/env python3

from pandocfilters import toJSONFilter, Math, Para
"""
Pandoc filter to convert gitlab flavored markdown to pandoc flavored markdown
"""


def gitlab_markdown(key, value, format, meta):
    if key == "CodeBlock":
        [[identification, classes, keyvals], code] = value
        if len(classes) > 0 and classes[0] == "math":
            fmt = {'t': 'DisplayMath',
                   'c': []}
            return Para([Math(fmt, code)])

    elif key == "Math":
        [fmt, code] = value
        if isinstance(fmt, dict) and fmt['t'] == "InlineMath":
            # if fmt['t'] == "InlineMath":
            return Math(fmt, code.strip('`'))


if __name__ == "__main__":
    toJSONFilter(gitlab_markdown)



from distutils.core import setup
import os


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(name='pandocfilters',
      version='1.4.3',
      description='Utilities for writing pandoc filters in python',
      long_description=read('README.rst'),
      author='John MacFarlane',
      author_email='fiddlosopher@gmail.com',
      url='http://github.com/jgm/pandocfilters',
      py_modules=['pandocfilters'],
      license="BSD-3-Clause",
      keywords=['pandoc'],
      python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Environment :: Console',
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Text Processing :: Filters',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
      ],
      )



#!/usr/bin/env python3

"""
Pandoc filter to process code blocks with class "ly" containing
Lilypond notation.  Assumes that Lilypond and Ghostscript are
installed, plus [lyluatex](https://github.com/jperon/lyluatex) package for
LaTeX, with LuaLaTeX.
"""

import os
from sys import getfilesystemencoding, stderr
from subprocess import Popen, call, PIPE
from hashlib import sha1
from pandocfilters import toJSONFilter, stringify,\
    Para, Image, RawInline, RawBlock

IMAGEDIR = "tmp_ly"
LATEX_DOC = """\\documentclass{article}
\\usepackage{libertine}
\\usepackage{lyluatex}
\\pagestyle{empty}
\\begin{document}
%s
\\end{document}
"""


def sha(x):
    return sha1(x.encode(getfilesystemencoding())).hexdigest()


def latex(code):
    """LaTeX inline"""
    return RawInline('latex', code)


def latexblock(code):
    """LaTeX block"""
    return RawBlock('latex', code)


def ly2png(lily, outfile, kvs):
    p = Popen([
        "lilypond",
        "-dno-point-and-click",
        "-dbackend=eps",
        "-djob-count=2",
        "-ddelete-intermediate-files",
        "-o", outfile,
        "-"
    ], stdin=PIPE, stdout=stderr)
    p.stdin.write(("\\paper{\n"
        "indent=0\\mm\n"
        "oddFooterMarkup=##f\n"
        "oddHeaderMarkup=##f\n"
        "bookTitleMarkup = ##f\n"
        "scoreTitleMarkup = ##f\n"
        "line-width = %s\n"
        "}\n"
        "#(set-global-staff-size %s)\n" % (
            kvs['width'][:-2] + '\\' + kvs['width'][-2:],
            kvs['staffsize']) +
        lily).encode("utf-8"))
    p.communicate()
    p.stdin.close()
    call([
        "gs",
        "-sDEVICE=pngalpha",
        "-r144",
        "-sOutputFile=" + outfile + '.png',
        outfile + '.pdf',
    ], stdout=stderr)


def png(contents, kvs):
    """Creates a png if needed."""
    outfile = os.path.join(IMAGEDIR, sha(contents + str(kvs['staffsize'])))
    src = outfile + '.png'
    if not os.path.isfile(src):
        try:
            os.mkdir(IMAGEDIR)
            stderr.write('Created directory ' + IMAGEDIR + '\n')
        except OSError:
            pass
        ly2png(contents, outfile, kvs)
        stderr.write('Created image ' + src + '\n')
    return src


def calc_params(kvs, meta):
    if 'staffsize' not in kvs:
        try:
            kvs['staffsize'] = int(
                meta['music']['c']['lilypond']['c']['staffsize']['c']
            )
        except (KeyError, TypeError):
            kvs['staffsize'] = 20
    if 'width' not in kvs:
        try:
            kvs['width'] = \
                meta['music']['c']['lilypond']['c']['width']['c'][0]['c']
        except (KeyError, TypeError):
            kvs['width'] = '210mm'
    return kvs


def lily(key, value, fmt, meta):
    if key == 'Code':
        [[ident, classes, kvs], contents] = value  # pylint:disable=I0011,W0612
        kvs = {key: value for key, value in kvs}
        if "ly" in classes:
            kvs = calc_params(kvs, meta)
            if fmt == "latex":
                if ident == "":
                    label = ""
                else:
                    label = '\\label{' + ident + '}'
                return latex(
                    '\\includely[staffsize=%s]{%s}' % (
                        kvs['staffsize'], contents
                    ) +
                    label
                )
            else:
                infile = contents + (
                    '.ly' if '.ly' not in contents else ''
                )
                with open(infile, 'r') as doc:
                    code = doc.read()
                return [
                    Image(['', [], []],
                        [],
                        [png(code, kvs), ""]
                    )
                ]
    if key == 'CodeBlock':
        [[ident, classes, kvs], code] = value
        kvs = {key: value for key, value in kvs}
        if "ly" in classes:
            kvs = calc_params(kvs, meta)
            if fmt == "latex":
                if ident == "":
                    label = ""
                else:
                    label = '\\label{' + ident + '}'
                return latexblock(
                    '\\lily[staffsize=%s]{%s}' % (kvs['staffsize'], code) +
                    label
                )
            else:
                return Para([
                    Image(
                        ['', [], []],
                        [],
                        [png(code, kvs), ""]
                    )
                ])

if __name__ == "__main__":
    toJSONFilter(lily)



#!/usr/bin/env python
from pandocfilters import toJSONFilter
import re

"""
Pandoc filter that causes everything between
'<!-- BEGIN COMMENT -->' and '<!-- END COMMENT -->'
to be ignored.  The comment lines must appear on
lines by themselves, with blank lines surrounding
them.
"""

incomment = False


def comment(k, v, fmt, meta):
    global incomment
    if k == 'RawBlock':
        fmt, s = v
        if fmt == "html":
            if re.search("<!-- BEGIN COMMENT -->", s):
                incomment = True
                return []
            elif re.search("<!-- END COMMENT -->", s):
                incomment = False
                return []
    if incomment:
        return []  # suppress anything in a comment

if __name__ == "__main__":
    toJSONFilter(comment)
